{% extends base_template %}
{% from "macros/box.html" import m_box_content with context %}

{% block content %}
  {%- block before_form %}
  {%- endblock %}

  {%- call m_box_content(view.title) %}
  <table class="table table-striped table-condensed" style="margin: 10px 0 15px 0;">
    <tbody>
    {%- for field in form %}
    {%- if field.data or field.flags.render_empty %}
      <tr class="row">
        <td class="col-sm-4">{{ field.label }}</td>
        <td class="col-sm-8" colspan="3">{{ field.render_view() | safe }}</td>
      </tr>
    {%- endif %}
    {%- endfor %}
    </tbody>
  </table>
  {%- endcall %}

  {%- block after_form %}
  {%- endblock %}
{% endblock %}
