{% extends "base.html" %}

{% from "macros/box.html" import m_box, m_box_menu with context %}

{% block main %}
  <div class="row">
    <div class="col-md-9">
      {% block content %}
      {% endblock %}
    </div>

    <div class="col-md-3">
      {%- call m_box_menu() %}
        <ul class="nav nav-pills nav-stacked">
          {%- for action in actions.for_category('navigation:admin:panel') %}
            <li class="{{ action.status }}">{{ action.render() }}</li>
          {%- endfor %}
        </ul>
      {%- endcall %}
    </div>
  </div>
{% endblock %}
