{#
  Macro for creating standardized buttons
#}

{%- macro a_button(btn_cls, url, label) %}
  <a class="btn btn-{{ btn_cls }}" href="{{ url }}"><i class="fa fa-plus"></i> {{ label }}</a>
{%- endmacro %}

{%- macro create_a(url, label) %}
  {{ a_button('primary', url, label) }}
{%- endmacro %}
