# coding=utf-8
"""Useful decorators for web views."""
from __future__ import absolute_import, division, print_function, \
    unicode_literals

# We're now using the `deprecated` decorator from the `python-deprecated`
# package. Direct use of this package is not recommended.
from deprecated import deprecated

__all__ = ('deprecated',)
